import ast
import numpy as np
import pandas as pd
import matplotlib
matplotlib.use('agg')
import matplotlib.pyplot as plt
from matplotlib.ticker import AutoLocator
import scipy.stats as stats
import math
# font = {'weight' : 'bold',
#         'size'   : 30}
# matplotlib.rc('font', **font)
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

NUM_COLORS = 10
cm = plt.get_cmap('tab10')
colors = [cm(1.*i/NUM_COLORS) for i in range(NUM_COLORS)]

def readNEOSresult(file):
	x1 = []
	w1 = []
	with open(file) as f:
	    content = f.readlines()
	xstart=False
	xend=False
	wstart=False
	wend=False
	for x in content:
		if x == ';\n' and xstart==True and xend==False:
			print('xend')
			xend=True
		if x == ';\n' and wstart==True and wend==False:
			print('wend')
			wend=True
		if xstart==True and xend==False:
			x1.append([float(i) for i in x.split()])
		if wstart==True and wend==False:
			w1.append([float(i) for i in x.split()])
		if ':=' in x and xstart==False:
			print('xstart')
			xstart=True
		if ':=' in x and 'y [*] :=' not in x and xend==True and wstart==False:
			print('wstart')
			wstart=True
	x1 = np.array(x1)[:,1:]
	w1 = np.array(w1)[:,1:]
	return (x1, w1)

def findquadrant(input_list):
    l = round(float(len(input_list))/4)
    return [l,2*l,3*l]

def add_arrow(line, position=None, direction='right', size=15, color=None):
    """
    add an arrow to a line.

    line:       Line2D object
    position:   x-position of the arrow. If None, mean of xdata is taken
    direction:  'left' or 'right'
    size:       size of the arrow in fontsize points
    color:      if None, line color is taken.
    """
    if color is None:
        color = line[0].get_color()

    xdata = line[0].get_xdata()
    ydata = line[0].get_ydata()

    start_inds = findquadrant(xdata)
    for start_ind in start_inds:
	    if direction == 'right':
	        end_ind = start_ind + 1
	    else:
	        end_ind = start_ind - 1

	    line[0].axes.annotate('',
	        xytext=(xdata[start_ind], ydata[start_ind]),
	        xy=(xdata[end_ind], ydata[end_ind]),
	        arrowprops=dict(arrowstyle="-|>", color=color),
	        size=size
	    )

angles = np.linspace(0,2.1*math.pi,100)
boundaries = np.array([[math.cos(angle),math.sin(angle)] for angle in angles])

# w1_colinear = pd.read_csv('w_colinear_w=1.5_eta=0.01.csv',header=None).values;
# w2_colinear = pd.read_csv('w_colinear_w=2.5_eta=0.01.csv',header=None).values;
# w3_colinear = pd.read_csv('w_colinear_w=1_eta=0.01.csv',header=None).values;
# # w4_colinear = pd.read_csv('w4_colinear.csv',header=None).values;


# w1_greedy = pd.read_csv('w_greedy_w=1.5_eta=0.01.csv',header=None).values;
# w2_greedy = pd.read_csv('w_greedy_w=2.5_eta=0.01.csv',header=None).values;
# w3_greedy = pd.read_csv('w_greedy_w=1_eta=0.01.csv',header=None).values;


# w3_NLOptControl = pd.read_csv('w0=(0,1)_NLOptControl.csv',header=None,delimiter='\t').values;
# w2_NLOptControl = pd.read_csv('w0=(0,2.5)_NLOptControl.csv',header=None,delimiter='\t').values;
# w1_NLOptControl = pd.read_csv('w0=(-1.5,0.5)_NLOptControl.csv',header=None,delimiter='\t').values;

# x1_colinear = pd.read_csv('X_colinear_w=1.5_eta=0.01.csv',header=None).values;
# x2_colinear = pd.read_csv('X_colinear_w=2.5_eta=0.01.csv',header=None).values;
# x3_colinear = pd.read_csv('X_colinear_w=1_eta=0.01.csv',header=None).values;


# x1_greedy = pd.read_csv('X_greedy_w=1.5_eta=0.01.csv',header=None).values;
# x2_greedy = pd.read_csv('X_greedy_w=2.5_eta=0.01.csv',header=None).values;
# x3_greedy = pd.read_csv('X_greedy_w=1_eta=0.01.csv',header=None).values;


# x3_NLOptControl = pd.read_csv('x_w0=(0,1)_NLOptControl.csv',header=None,delimiter='\t').values;
# x2_NLOptControl = pd.read_csv('x_w0=(0,2.5)_NLOptControl.csv',header=None,delimiter='\t').values;
# x1_NLOptControl = pd.read_csv('x_w0=(-1.5,0.5)_NLOptControl.csv',header=None,delimiter='\t').values;

w1_colinear = pd.read_csv('results/w0=(-1.5,0.5)_STRAIGHT_w.csv',header=None).values;
w2_colinear = pd.read_csv('results/w0=(0,2.5)_STRAIGHT_w.csv',header=None).values;
w3_colinear = pd.read_csv('results/w0=(0,1)_STRAIGHT_w.csv',header=None).values;

w1_greedy = pd.read_csv('results/w0=(-1.5,0.5)_GREEDY_w.csv',header=None).values;
w2_greedy = pd.read_csv('results/w0=(0,2.5)_GREEDY_w.csv',header=None).values;
w3_greedy = pd.read_csv('results/w0=(0,1)_GREEDY_w.csv',header=None).values;

w1_DP = pd.read_csv('results/w0=(-1.5,0.5)_NLP_w_eta=0.02.csv',header=None,delimiter='\t').values;
w2_DP = pd.read_csv('results/w0=(0,2.5)_NLP_w_eta=0.02.csv',header=None,delimiter='\t').values;
w3_DP = pd.read_csv('results/w0=(0,1)_NLP_w_eta=0.02.csv',header=None,delimiter='\t').values;

w1_NLOptControl = pd.read_csv('results/w0=(-1.5,0.5)_CNLP_w.csv',header=None,delimiter='\t').values;
w2_NLOptControl = pd.read_csv('results/w0=(0,2.5)_CNLP_w.csv',header=None,delimiter='\t').values;
w3_NLOptControl = pd.read_csv('results/w0=(0,1)_CNLP_w.csv',header=None,delimiter='\t').values;


x1_colinear = pd.read_csv('results/w0=(-1.5,0.5)_STRAIGHT_x.csv',header=None).values;
x2_colinear = pd.read_csv('results/w0=(0,2.5)_STRAIGHT_x.csv',header=None).values;
x3_colinear = pd.read_csv('results/w0=(0,1)_STRAIGHT_x.csv',header=None).values;

x1_greedy = pd.read_csv('results/w0=(-1.5,0.5)_GREEDY_x.csv',header=None).values;
x2_greedy = pd.read_csv('results/w0=(0,2.5)_GREEDY_x.csv',header=None).values;
x3_greedy = pd.read_csv('results/w0=(0,1)_GREEDY_x.csv',header=None).values;

x1_DP = pd.read_csv('results/w0=(-1.5,0.5)_NLP_x_eta=0.02.csv',header=None,delimiter='\t').values;
x2_DP = pd.read_csv('results/w0=(0,2.5)_NLP_x_eta=0.02.csv',header=None,delimiter='\t').values;
x3_DP = pd.read_csv('results/w0=(0,1)_NLP_x_eta=0.02.csv',header=None,delimiter='\t').values;

x1_NLOptControl = pd.read_csv('results/w0=(-1.5,0.5)_CNLP_x.csv',header=None,delimiter='\t').values;
x2_NLOptControl = pd.read_csv('results/w0=(0,2.5)_CNLP_x.csv',header=None,delimiter='\t').values;
x3_NLOptControl = pd.read_csv('results/w0=(0,1)_CNLP_x.csv',header=None,delimiter='\t').values;

# x1_DP, w1_DP = readNEOSresult('RLS_result_w1.5_eta=0.01.txt')
# x2_DP, w2_DP = readNEOSresult('RLS_result_w2.5_eta=0.01.txt')
# x3_DP, w3_DP = readNEOSresult('RLS_result_w1_eta=0.01.txt')

def setlims():
    plt.xlim(xmax = 1.1)
    plt.xlim(xmin = -1.1)
    plt.ylim(ymax = 0.5)
    plt.ylim(ymin = -1)
    #plt.grid()

fig = plt.figure(figsize=(3.25,3))

frame = plt.subplot(221)
frame.axes.xaxis.set_ticklabels([])
plt.plot(boundaries[:,0],boundaries[:,1], '--',color = (0.8,0.8,0.8))
line1 = plt.plot(x1_greedy[0,::10], x1_greedy[1,::10], 'o', fillstyle = 'none', markersize=2.0, label = 'GREEDY', color = colors[0])
add_arrow(line1)
plt.legend(loc="upper left",fontsize="x-small")   # make legend text smaller
plt.xticks(fontsize=6)
plt.yticks(fontsize=6)
plt.axis("equal")
setlims()

frame = plt.subplot(222)
frame.axes.yaxis.set_ticklabels([])
frame.axes.xaxis.set_ticklabels([])
plt.xlim(xmax = 1.1)
plt.xlim(xmin = -0.5)
plt.ylim(ymax = 0.5)
plt.ylim(ymin = -1.1)
plt.plot(boundaries[:,0],boundaries[:,1], '--',color = (0.8,0.8,0.8))
line2 = plt.plot(x1_colinear[0,::10], x1_colinear[1,::10], 'o', fillstyle = 'none', markersize=2, label = 'STRAIGHT', color = colors[1])
add_arrow(line2)
plt.legend(loc="upper left",fontsize="x-small")   # make legend text smaller
plt.xticks(fontsize=6)
plt.yticks(fontsize=6)
plt.axis("equal")
setlims()

plt.subplot(223)
plt.xlim(xmax = 1.1)
plt.xlim(xmin = -0.5)
plt.ylim(ymax = 0.5)
plt.ylim(ymin = -1.1)
plt.plot(boundaries[:,0],boundaries[:,1], '--',color = (0.8,0.8,0.8))
line3 = plt.plot(x1_DP[::10,0], x1_DP[::10,1], 'o', fillstyle = 'none', markersize=2, label = 'NLP', color = colors[2])
add_arrow(line3)
plt.legend(loc="upper left",fontsize="x-small")   # make legend text smaller
plt.xticks(fontsize=6)
plt.yticks(fontsize=6)
plt.axis("equal")
setlims()

frame = plt.subplot(224)
frame.axes.yaxis.set_ticklabels([])
plt.xlim(xmax = 1.1)
plt.xlim(xmin = -0.5)
plt.ylim(ymax = 0.5)
plt.ylim(ymin = -1.1)
plt.plot(boundaries[:,0],boundaries[:,1], '--',color = (0.8,0.8,0.8))
line4 = plt.plot(x1_NLOptControl[:,0], x1_NLOptControl[:,1], '--', fillstyle = 'none', linewidth=2, label = 'CNLP', color = colors[3])
add_arrow(line4)
plt.legend(loc="upper left",fontsize="x-small")   # make legend text smaller
plt.xticks(fontsize=6)
plt.yticks(fontsize=6)
plt.axis("equal")
setlims()

# plt.plot(-1.5,0.5, 'o', markersize=8.0, label = '$w(0)$')
# plt.plot(1,0, '*', markersize=8.0, label = '$w^*$')

plt.tight_layout(pad=0) 

fig.savefig('control2_no_type3.pdf')
plt.close()
